/*
   generate daten.def for tilings,  Dez98 JS
   
   compile: g++ -Wall -o a pic2.c -lm
   use:     a.out ?nr? ?e1x? ?e1y? ?e2x? ?e2y?
            nr: 0..11
   example: a.out 4  2 1  1 2		# volume=Det((2,1),(1,2))=3
            a.out 4  2 0  0 2		# volume=4
   
   edgevectors mcf:  1,0,0,1,  1,1,1,-1,  2,1,1,2,  2,0,0,2,  2,1,-1,2,  
               utlf: 1,0,0,1,             3,0,2,1,  2,0,0,2,
               (most compact form, upper triangular lattice form) 
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define Nb 4		/* Number of base vectors (coordinate-axes) all points are integer numbers! */ 
int typ=0;

double xl=10.01,   	/* plot window: 0..xl,0..yl */
       yl=10.01,
       xo=0.0,		/* offset */
       yo=0.0,
       b[Nb][2]={{0,1},	/* basevector e_x */
                 {1,0},   /* basevector e_y */
                 {0,0},   /* basevector reserved for mixed lattices */
                 {0,0}};	/* basevector reserved for mixed lattices */

/* --- line in the box? ret:0=in, 2=out, 1=cut */
int cutline(double x1,double y1,double x2,double y2){
  int r=0;
  if(x1<-0.01 || x1>xl || y1<-0.01 || y1>yl) r++;
  if(x2<-0.01 || x2>xl || y2<-0.01 || y2>yl) r++;
  return r;
}

/* --- output for gnuplot!? nb = NumBases used 
   --- base can have more than 2 vectors */
int plotline(int *a,int *d,int nb){
  double x1,y1,x2,y2; int i;
  for(x1=y1=i=0;i<nb;i++){
    x1+=a[i]*b[i][0];	/* <a||base> 1.Punkt */
    y1+=a[i]*b[i][1];
  }
  for(x2=x1,y2=y1,i=0;i<nb;i++){
    x2+=d[i]*b[i][0];	/* <a+na||base> 2.Punkt */
    y2+=d[i]*b[i][1];
  }
  if(cutline(x1,y1,x2,y2)==0)
    if( typ==0 )
    printf(" %8.4f %8.4f\n %8.4f %8.4f\n\n",
           x1+xo,y1+yo,x2+xo,y2+yo);
    else
    printf(" %8.4f %8.4f %8.4f %8.4f\n",
           x1+xo,y1+yo,x2+xo,y2+yo);		/* for awk => xfig */
  /* --- gnuplot plot "<a.out" with linespoints */
  return 0;
}

int get_vol(int a[2][2]){	/* volume=det(a) */
   return(a[0][0]*a[1][1]-a[0][1]*a[1][0]); }

int sprod(int a[2], int b[2]){	/* spat-product */
   return a[0]*b[0]+a[1]*b[1]; }

int product(int a[2][2],int u[2][2],int au[2][2]){ /* au=a*u */
   int i,j,b[2][2]={{0,0},{0,0}};	/* puffer falls &prod=&factor */
   for(i=0;i<2;i++)for(j=0;j<2;j++)
   b[i][j]=a[i][0]*u[0][j]+a[i][1]*u[1][j];
   for(i=0;i<2;i++)for(j=0;j<2;j++)
   au[i][j]=b[i][j];
   return 0;
}
/* 2D-rotations det(T)=+1 oder als Permutation von sites tabellieren? */
int rotate[2][2][2]={
  {{1, 0}, { 0, 1}},
  {{0, 1}, {-1, 0}},
};         
/* 2D-reflections det(T)=+1 oder als Vectoren \pm 1 */
int reflect[2][2][2]={
 {{ 1, 0}, { 0, 1}},
 {{-1, 0}, { 0,-1}},
};         

/* TransponiertAdjuncte=det(A)*A^(-1); -> Bildung der Inversen Matrix */
int adjunct(int a[2][2],int adjtrans[2][2]){
   int b[2][2];		/* puffer, falls adjtrans==a */
   b[0][0]= a[1][1];	/* adjungierte 2x2 matrix => trivial */
   b[0][1]=-a[1][0];
   b[1][0]=-a[0][1];
   b[1][1]= a[0][0];
   adjtrans[0][0]= b[0][0];	/* transponieren */
   adjtrans[0][1]= b[1][0];
   adjtrans[1][0]= b[0][1];
   adjtrans[1][1]= b[1][1];
   return 0;
}          

int compare(int a1[2][2], int a2[2][2]){
    int bb1[2][2],bb2[2][2],i1,i2,v;
    /* compare  A1*T_i*A2^-1=int-matrix? */
    /* rechts - links Reihenfolge beachten! */
    v=abs(get_vol(a2));
    for(i1=0;i1<2*2;i1++){	/* all rotations+reflections */
       adjunct(a2,bb1);
       product( rotate[i1/2],bb1,bb2);
       product(reflect[i1%2],bb2,bb1);
       product(  a1,         bb1,bb2);
       for(i2=0;i2<2*2;i2++)if(abs(bb2[i2/2][i2%2])%(v)!=0)break;
       if(i2>=2*2){
         printf("# V=%2u ROT=%d REF=%d U=(%d %d, %d %d)\n",
           v,i1/2,i1%2,
           bb2[0][0]/v,bb2[0][1]/v,
           bb2[1][0]/v,bb2[1][1]/v);
         break;
       }   
#if 0
       fprintf(stderr,"# %d %d %d %d = %d %d %d %d\n",
               a1[0][0],a1[0][1],a1[1][0],a1[1][1],
               a2[0][0],a2[0][1],a2[1][0],a2[1][1]);
#endif
    }      
    if(i1==2*2) { /* printf("# a1!=a2\n"); */ return -1;}
    return 0;
}

int minmax(int e[2][2],int emin[2],int emax[2]){
   /* einhuellenden Kubus ermitteln (alle eck-koords checken) */
   int i,j,k;
   for(i=0;i<2;i++){                     
     emin[i]=emax[i]=e[0][i]+e[1][i];    
     if(emin[i]>0)emin[i]=0;
     if(emax[i]<0)emax[i]=0;
     for(j=0;j<2;j++){
       if(e[j][i]<emin[i])emin[i]=e[j][i];
       if(e[j][i]>emax[i])emax[i]=e[j][i];
       for(k=j+1;k<2;k++){
         if(e[j][i]+e[k][i]<emin[i])emin[i]=e[j][i]+e[k][i];
         if(e[j][i]+e[k][i]>emax[i])emax[i]=e[j][i]+e[k][i];
   }}}                                   
   printf("# emin=(%d %d) emax=(%d %d)\n", 
          emin[0],emin[1],emax[0],emax[1]);     
   return 0;
}
/*                                       
+++TeX+++                                

 Beispiel in zwei Dimensionen. $a=(a_x,a_y)$ und $b=(b_x,b_y)$ sind die
 zwei Vektoren, die die Elementarzelle aufspannen.
 Sie sind im allgemeinen nicht senkrecht zueinander.
 Der Vektor $r=(r_x,r_y)$ liegt innerhalb der Elementarzelle, wenn seine
 Komponenten $(r_a,r_b)$ bezueglich der Elementarvektoren zwischen 0 und 1
 liegen.                                 
 \begin{eqnarray*}                       
      r &=& r_a a   + r_b b \\           
 %                                       
    a r &=& r_a a^2 + r_b a b \\         
    b r &=& r_a a b + r_b b^2 \\         
 %                                       
    r_a &=& \frac{(b^2a-(a,b)b)r}{a^2b^2-(a,b)^2} \\
    r_b &=& \frac{(a^2b-(a,b)a)r}{a^2b^2-(a,b)^2} \\
 \end{eqnarray*}                         
 Grenzfall $(a,b)=0$ liefert bekannte Beziehungen.
---TeX--- */                             

/* test if R lying in a parallel epiped, move r to x (by n*e) */
int inside(int r[2],int e[2][2],int x[2]){
   int i,aa,ab,bb,det,ra,rb,ar,br,ret=0;
   i=0;
   aa=sprod(e[0],e[0]);
   ab=sprod(e[0],e[1]);
   bb=sprod(e[1],e[1]);
   det= bb*aa - ab*ab;
   ar=sprod(e[0],r);
   br=sprod(e[1],r);    
   ra= ( + bb*ar - ab*br );	/* det * projection of a !? */
   rb=-( + ab*ar - aa*br );
   if(ra>=0 && rb>=0 && ra<det && rb<det) ret=1; /* ok, its lying inside */
   						/* else R is outside of E */
   if(x){
     x[0]=r[0];
     x[1]=r[1];
     if(ra<0)ra=-((-ra-1)/det)-1;else ra/=det;
     if(rb<0)rb=-((-rb-1)/det)-1;else rb/=det;
     x[0]-=ra*e[0][0]+rb*e[1][0];           
     x[1]-=ra*e[0][1]+rb*e[1][1];           
   }
#if 0
   printf("# inside r(%p)= % d % d  rr(%p)=% d % d  ret=%d\n",
     r,r[0],r[1],x,((x)?x[0]:0),((x)?x[1]:0),ret);
#endif
   return ret;
}

/* ---------------------------------------------------------------
   Nb=num_of_base_vectors=4
   b=base_vectors 4*(x,y) default={{1,0},{0,1},{0,0},{0,0}}
   u=unit_vectors u1,u2 in units of b u[2][Nb]
   p=points_of_unitcell in units of b p[npc][Nb]
   w=bonds_of_unitcell  { site1, site2, J, R2+u1, R2+u2 }
                                           R + PBC
   npc=sites_per_unitcell 
   wpc=bonds_per_unitcell
   e1,e2=edgevectors_of_clster in units of unit_vector u1,u2 --- enough? ---
   --------------------------------------------------------------- */
int outdef(double b[Nb][2], int u[2][Nb],
           int npc, int p[][Nb],
           int wpc, int w[][5], int e[2][2]){
    int i,j,k,n,s1,s2,v,t[2][2],emin[2],emax[2],r[2],rr[2],r2[2];
    double x1,y1,x2,y2;

    v=get_vol(e);

    /* ---=== generate upper triangular matrix ===--- */
    t[1][0]=0; i=1;
    for(t[1][1]=1; t[1][1]<=v;      t[1][1]++)
    for(t[0][0]=1; t[0][0]<=v;      t[0][0]++) if( t[1][1]*t[0][0]==v )
    for(t[0][1]=0; t[0][1]<t[1][1]; t[0][1]++)
    {	/* test e*T*t^1=U unitaere matrix => same lattice */
       if(!compare(e,t)) { i=0; goto label1;}	/* break */
    }  label1:	
    /* label1: Do you know a better way leaving double loops? */

    printf("# utlf %3d %3d %3d %3d\n",t[0][0],t[0][1],t[1][0],t[1][1]);
    if(i){ fprintf(stderr,"# error: utlf not found\n"); }
 
    minmax(e,emin,emax);
    printf("# base:");
    for(i=0;i<Nb;i++)if(b[i][0]!=0 || b[i][1]!=0)
    printf(" b%d=(% .3f,% .3f)",i+1,b[i][0],b[i][1]);
    printf("\n");
    printf("#===base=== 1 0 0 1\n");
    x1=y1=x2=y2=0.0;		/* unitcell */
    for(i=0;i<Nb;i++){ x1+=u[0][i]*b[i][0]; y1+=u[0][i]*b[i][1];
                       x2+=u[1][i]*b[i][0]; y2+=u[1][i]*b[i][1]; }
    printf("# geometrical volume of unitcell % .6f\n",x1*y2-y1*x2);
    printf("#===edge=== %.3f %.3f %.3f %.3f\n",	/* t==e */
			e[0][0]*x1+e[0][1]*x2,
                        e[0][0]*y1+e[0][1]*y2,
                        e[1][1]*x2+e[1][0]*x1,
                        e[1][1]*y2+e[1][0]*y1);
    /* frame for PBC */
    printf("#===pbcf=== 0 0 %.3f %.3f %.3f %.3f %.3f %.3f 0 0\n",
 			e[0][0]*x1+e[0][1]*x2,
                        e[0][0]*y1+e[0][1]*y2,
 			e[0][0]*x1+e[0][1]*x2+e[1][1]*x2+e[1][0]*x1,
                        e[0][0]*y1+e[0][1]*y2+e[1][1]*y2+e[1][0]*y1,
                        e[1][1]*x2+e[1][0]*x1,
                        e[1][1]*y2+e[1][0]*y1);
    printf("#+++model+++\n");
    printf("# NN=Number_of_sites Nw=Number_of_bonds Nv=Number_of_bonds(2)\n");
    printf("%d %d 0\n",npc*v,wpc*v);
    printf("# s  X  Y  Z p_e p_U (s=siteindex  (X Y Z)=position  p=index of parameter)\n");
    /* --- alle punkte abwandern a1*b1+a2*b2 */
    i=0;	/* unitcell counter, write to buffer num[][]=i */
    for(r[1]=emin[1]; r[1]<emax[1]; r[1]++)	/* all unitcells */
    for(r[0]=emin[0]; r[0]<emax[0]; r[0]++)	/* lying in e[2]-epiped */
    if( inside(r,e,(int *)NULL) ){
     for(n=0; n<npc; n++)
     { 
       x1=0;
       y1=0;
       for(j=0;j<Nb;j++){ 
         x1+=r[0]*u[0][j]*b[j][0]
            +r[1]*u[1][j]*b[j][0]
            +     p[n][j]*b[j][0];
         y1+=r[0]*u[0][j]*b[j][1]
            +r[1]*u[1][j]*b[j][1]
            +     p[n][j]*b[j][1];
       }
       printf(" %2d % 5.3f % 5.3f 0 8 7\n",n+npc*i,x1,y1);
     }
     i++;
    }

    /* Nachbarn, one unitcell is multiplied */
    printf("# Bond s1 s2 p_t p_J p_V\n");
    /* --- alle Elementarzellen abwandern a1*b1+a2*b2 */
    i=0;
    for(r[1]=emin[1]; r[1]<emax[1]; r[1]++)	/* all unitcells */
    for(r[0]=emin[0]; r[0]<emax[0]; r[0]++)	/* lying in e[2]-epiped */
    if( inside(r,e,(int *)NULL) ){
     for(n=0; n<wpc; n++)
     { 
      rr[0]=r[0]+w[n][3];
      rr[1]=r[1]+w[n][4];	/* neighbourcell */
      j=i;k=1;
      if( w[n][3]!=0 || w[n][4]!=0 ){		/* s2 in other cell */
        /* move rr inside of e */
        k=inside(rr,e,rr);
        /* this is not the fastest solution to find the number of unitcell2 */
        j=0;
        for(r2[1]=emin[1]; r2[1]<emax[1]; r2[1]++)	/* all unitcells */
        for(r2[0]=emin[0]; r2[0]<emax[0]; r2[0]++)	/* lying in e[2]-epiped */
        if( inside(r2,e,(int *)NULL) ) {
          if( rr[0]==r2[0] && rr[1]==r2[1] ) goto label2; /* no better way? */
          j++;
        } label2: {}
      }
      s1=w[n][0]+npc*i;
      s2=w[n][1]+npc*j;
      /* Richtungsvektor fuer PBC angeben! 0..+9 = 0,n,s,e,w,ne,nw,se,sw */
      if(!k) printf(" %2d %2d %2d  0 %+2d  0\n",i*wpc+n,s1,s2,w[n][2]);
      else   printf(" %2d %2d %2d  0 % 2d  0\n",i*wpc+n,s1,s2,w[n][2]);
     }
     i++;
    }
    printf("#---model---\n");
    return 0;
}


int main(int argn, char *argv[]){
  int a[Nb],d[Nb],a1,a2,i1,i2,v,vvv=0;
  int e[2][2]={{1,0},{0,1}};
  if(argn>1) vvv=atoi(argv[1]);
  if(argn>5){ e[0][0]=atoi(argv[2]); e[0][1]=atoi(argv[3]);
              e[1][0]=atoi(argv[4]); e[1][1]=atoi(argv[5]);
  }
  if(argn>6) typ=atoi(argv[6]);
  v=get_vol(e);			/* volume of cluster */
  if( typ==0 ){
    printf("# def-file, tiling%d e= %d %d %d %d  v=%d\n"
           "# automaticly generated by pic2 v0.1 Feb99 JS\n",
           vvv,e[0][0],e[0][1],e[1][0],e[1][1],v);
  }
  if( v<=0 ){ printf("#error: volume<=0\n");return -1; }
  
  if(vvv==0){ 
    int npc=1;				/* 2 sites per unitcell */
    int wpc=6*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
                      { cos(M_PI/3), sin(M_PI/3) }, {0,0}, {0,0} };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 0, 0, 0 },
			{ -1, 1, 0, 0 } };
    int p[1][Nb]={			/* R_i of 6 sites in units of base */
			{  0, 0, 0, 0 } };
    int w[6*1/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 0, 1,  1,  0},  
      { 0, 0, 1,  0,  1},  /*   2nd site is e[1] PBC located */
      { 0, 0, 1,  1,  1}}; /*   2nd site is e[0]+e[1] PBC located */
  
    printf("# triangular lattice 3.3.3.3.3.3\n");
    outdef( b, u, npc, p, wpc, w, e);

  }
  
  if(vvv==1){ 
    int npc=1;				/* 1 sites per unitcell */
    int wpc=4*npc/2; 
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ { 1, 0 },
                      { 0, 1 }, {0,0}, {0,0} };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 0, 0, 0 },
			{  0, 1, 0, 0 } };
    int p[1][Nb]={			/* R_i of 6 sites in units of base */
			{  0, 0, 0, 0 }}; 
    int w[4*1/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 0, 1,  0,  1},  /*   2nd site is e[1] PBC located */
      { 0, 0, 1,  1,  0}}; /*   2nd site is e[0] PBC located */
  
    printf("# rectangular lattice 4.4.4.4\n");
    outdef( b, u, npc, p, wpc, w, e);

  }
  
  if(vvv==2){ 
    int npc=2;				/* 2 sites per unitcell */
    int wpc=3*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
                      { cos(M_PI/3), sin(M_PI/3) }, {0,0}, {0,0} };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 1, 0, 0 },
			{ -1, 2, 0, 0 } };
    int p[2][Nb]={			/* R_i of 6 sites in units of base */
			{  0, 1, 0, 0 },
			{  0, 2, 0, 0 } };
    int w[3*2/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 1,  0,  0},
      { 0, 1, 1,  0, -1},  /*   2nd site is -e[1] PBC located */
      { 0, 1, 1, -1,  0}}; /*   2nd site is -e[1] PBC located */
  
    printf("# honeycomb lattice 6.6.6\n");
    outdef( b, u, npc, p, wpc, w, e);

  }

  if(vvv==3){ 
    int npc=6;				/* 6 sites per unitcell */
    int wpc=5*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,            0 },
                      { cos(M_PI/3), -sin(M_PI/3) }, {0,0}, {0,0} };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  3,-1, 0, 0 },
			{ -2, 3, 0, 0 } };
    int p[6][Nb]={			/* R_i of 6 sites in units of base */
			{  1, 0, 0, 0 },
			{  2, 0, 0, 0 },
			{  0, 1, 0, 0 },
			{  1, 1, 0, 0 },
			{ -1, 2, 0, 0 },
			{  0, 2, 0, 0 } };
    int w[5*6/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 1,  0,  0}, /* 1st triangle */
      { 1, 3, 1,  0,  0},
      { 3, 0, 1,  0,  0}, 
      { 2, 5, 1,  0,  0}, /* 2nd triangle */
      { 5, 4, 1,  0,  0},
      { 4, 2, 1,  0,  0},
      { 0, 2, 2,  0,  0}, /* hexagon */
      { 2, 1, 2, -1,  0}, /*   2nd site is -e[0] PBC located */
      { 1, 5, 2,  0, -1}, /*   2nd site is -e[1] PBC located */
      { 5, 3, 2,  0,  0},
      { 3, 4, 2,  1,  0}, 
      { 4, 0, 2,  0,  1},
      { 0, 5, 3,  0, -1}, /* dimer,  2nd site is -e[1] PBC located */
      { 1, 4, 3,  1,  0},          
      { 2, 3, 3,  0,  0}};
  
    printf("# 3.3.3.3.6  J1=triangle, J2=hexagon, J3=dimer\n");
    outdef( b, u, npc, p, wpc, w, e);

  }

  if(vvv==4){ 
    int npc=6;				/* 6 sites per unitcell */
    int wpc=5*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
                       { cos(M_PI/3), sin(M_PI/3) }, {0,0}, {0,0} };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  3,-1, 0, 0 },
			{ -2, 3, 0, 0 } };
    int p[6][Nb]={			/* R_i of 6 sites in units of base */
			{  1, 0, 0, 0 },
			{  2, 0, 0, 0 },
			{  0, 1, 0, 0 },
			{  1, 1, 0, 0 },
			{ -1, 2, 0, 0 },
			{  0, 2, 0, 0 } };
    int w[5*6/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 1,  0,  0}, /* 1st triangle */
      { 1, 3, 1,  0,  0},
      { 3, 0, 1,  0,  0}, 
      { 2, 5, 1,  0,  0}, /* 2nd triangle */
      { 5, 4, 1,  0,  0},
      { 4, 2, 1,  0,  0},
      { 0, 2, 2,  0,  0}, /* hexagon */
      { 2, 1, 2, -1,  0}, /*   2nd site is -e[0] PBC located */
      { 1, 5, 2,  0, -1}, /*   2nd site is -e[1] PBC located */
      { 5, 3, 2,  0,  0},
      { 3, 4, 2,  1,  0}, 
      { 4, 0, 2,  0,  1},
      { 0, 5, 3,  0, -1}, /* dimer,  2nd site is -e[1] PBC located */
      { 1, 4, 3,  1,  0},          
      { 2, 3, 3,  0,  0}};
  
    printf("# 3.3.3.3.6  J1=triangle, J2=hexagon, J3=dimer\n");
    outdef( b, u, npc, p, wpc, w, e);

  }

  if(vvv==5){ 
    int npc=2;				/* 2 sites per unitcell */
    int wpc=5*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      { cos(M_PI/3), sin(M_PI/3) }, {0,0} };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 0, 0, 0 },
			{  0, 1, 1, 0 } };
    int p[2][Nb]={			/* R_i of 2 sites in units of base */
			{  0, 0, 0, 0 },
			{  0, 0, 1, 0 } };
    int w[5*2/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 0, 2,  1,  0}, /*   2nd site is  e[0] PBC located */
      { 1, 1, 2,  1,  0}, /*   2nd site is  e[0] PBC located */
      { 0, 1, 3,  0,  0},
      { 0, 1, 3, -1,  0}, 
      { 0, 1, 1,  0, -1}};
  
    printf("# 3.3.3.4.4 trellis J1=steps J2=lines J3=zickzack\n");
    outdef( b, u, npc, p, wpc, w, e);

  }

  if(vvv==6){ 
    int npc=4;				/* 4 sites per unitcell */
    int wpc=5*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      {-cos(M_PI/3), sin(M_PI/3) },
                      { cos(M_PI/6), sin(M_PI/6) } };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 0, 0, 1 },
			{  0, 1, 1, 0 } };
    int p[4][Nb]={			/* R_i of 2 sites in units of base */
			{  0, 0, 0, 0 },
			{  0, 1, 0, 0 },
			{  1, 1, 0, 0 },
			{  1, 1, 1, 0 } };
    int w[5*4/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 1,  0,  0}, /* square */
      { 1, 2, 1,  0,  0},
      { 2, 3, 1,  0, -1}, /*   2nd site is -e[1] PBC located */
      { 3, 0, 1,  0,  1},
      { 0, 2, 2, -1,  0},
      { 1, 3, 2,  0,  0},
      { 2, 3, 1,  0,  0},
      { 1, 0, 1,  0,  1},
      { 2, 1, 1,  1,  0}, 
      { 3, 0, 1,  1,  1}};
    printf("# 3.3.4.3.4 J1=square/triangle J2=triangle/triangle\n");
    outdef( b, u, npc, p, wpc, w, e);
  }


  if(vvv==7){ 
    int npc=6;				/* 4 sites per unitcell */
    int wpc=4*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      { cos(M_PI/3), sin(M_PI/3) },	/* .5, .866 */
                      { cos(M_PI/6), sin(M_PI/6) } };   /* .866, .5 */
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 0, 1, 1 },
			{ -1, 1, 2, 0 } };
    int p[6][Nb]={			/* R_i of 2 sites in units of base */
			{  0, 0, 1, 0 },
			{  0, 1, 1, 0 },
			{  0, 0, 1, 1 },
			{  0, 0, 2, 1 },
			{  0, 1, 2, 0 },
			{  0, 1, 2, 1 } };
    int w[6*4/2][5]={
      { 0, 1, 2,  0,  0}, /* triangle */
      { 1, 2, 2,  0,  0},
      { 2, 0, 2,  0,  0},
      { 3, 4, 2,  0,  0},
      { 4, 5, 2,  0,  0},
      { 5, 3, 2,  0,  0},
      { 0, 4, 1,  0, -1}, /* hexagon */
      { 4, 1, 1,  0,  0},
      { 1, 5, 1, -1,  0}, /*   2nd site is -e[0] PBC located */
      { 5, 2, 1,  0,  1},
      { 2, 3, 1,  0,  0}, 
      { 3, 0, 1,  1,  0}};
    printf("# 3.4.6.4 J1=hexagons J2=triangles\n");
    outdef( b, u, npc, p, wpc, w, e);
  }

  if(vvv==8){ 
    int npc=3;				/* 4 sites per unitcell */
    int wpc=4*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      { cos(M_PI/3), sin(M_PI/3) },
                      { cos(M_PI/6), sin(M_PI/6) } };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  2, 0, 0, 0 },
			{  0, 0, 2, 0 } };
    int p[3][Nb]={			/* R_i of 2 sites in units of base */
			{  1, 0, 0, 0 },
			{  0, 0, 1, 0 },
			{  1, 0, 1, 0 } };
    int w[3*4/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 1,  0,  0}, /* square */
      { 1, 2, 1,  0,  0},
      { 2, 0, 1,  0,  0}, /*   2nd site is -e[1] PBC located */
      { 2, 0, 1,  0,  1},
      { 0, 1, 1,  1, -1},
      { 1, 2, 1, -1,  0}};
    printf("# Kagom\\'e 3.6.3.6\n");
    outdef( b, u, npc, p, wpc, w, e);
  }

  if(vvv==9){ 
    int npc=6;				/* 6 sites per unitcell */
    int wpc=3*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      { cos(M_PI/3), sin(M_PI/3) },	/* 0.5, 0.866 */
                      { cos(M_PI/6), sin(M_PI/6) } };   /* 0.866, 0.5 */
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  2,-1, 0, 2 },
			{  0, 1, 2, 1 } };
    int p[6][Nb]={			/* R_i of 2 sites in units of base */
			{  1, 0, 0, 1 },
			{  0, 0, 1, 1 },
			{  1, 0, 1, 1 },
			{  1, 0, 1, 2 },
			{  1, 0, 2, 2 },
			{  2, 0, 1, 2 } };
    int w[3*6/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 2,  0,  0}, /* triangle */
      { 1, 2, 2,  0,  0},
      { 2, 0, 2,  0,  0}, /*   2nd site is -e[1] PBC located */
      { 3, 4, 2,  0,  0}, /* triangle */
      { 4, 5, 2,  0,  0},
      { 5, 3, 2,  0,  0}, /*   2nd site is -e[1] PBC located */
      { 2, 3, 1,  0,  0}, /* dodecahedron */
      { 5, 1, 1,  1,  0},
      { 0, 4, 1,  0, -1}}; /*   2nd site is -e[1] PBC located */
    printf("# 3.12.12  J1=dodecahedron J2=triangle\n");
    outdef( b, u, npc, p, wpc, w, e);
  }

  if(vvv==10){ 
    int npc=12;				/* 6 sites per unitcell */
    int wpc=3*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      { cos(M_PI/3), sin(M_PI/3) },	/* .5, .866 */
                      { cos(M_PI/6), sin(M_PI/6) } };	/* .866, .5 */
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  2,-3,-1, 3 },
			{ -1, 3, 2, 0 } };
    int p[12][Nb]={			/* R_i of 2 sites in units of base */
			{  1,-1, 0, 1 },
			{  1,-2, 0, 2 },
			{  1,-2, 0, 3 },
			{  1,-1, 0, 3 },
			{  1, 0, 0, 2 },
			{  1, 0, 0, 1 },
			{  0, 0, 1, 1 },
			{  0, 0, 1, 2 },
			{  0, 1, 1, 2 },
			{  0, 2, 1, 1 },
			{  0, 2, 1, 0 },
			{  0, 1, 1, 0 } };
    int w[12*3/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 1, 2, 1,  0,  0}, /* dodecahedron/hexagon */
      { 3, 4, 1,  0,  0},
      { 5, 0, 1,  0,  0},
      { 7, 8, 1,  0,  0},
      { 9,10, 1,  0,  0},
      {11, 6, 1,  0,  0},
      { 0, 1, 2,  0,  0}, /* hexagon/square */
      { 2, 3, 2,  0,  0},
      { 4, 5, 2,  0,  0},
      { 6, 7, 2,  0,  0},
      { 8, 9, 2,  0,  0},
      {10,11, 2,  0,  0},
      { 0, 9, 3,  0, -1}, /* dodecahedron/square */
      {10, 3, 3, -1,  0},
      { 4, 7, 3,  0,  0}, /*   2nd site is -e[0] PBC located */
      { 8, 1, 3,  0,  1},
      { 2,11, 3,  1,  0},
      { 6, 5, 3,  0,  0}};
    printf("# 4.6.12  J1=dode/hexa J2=hexa/square J3=dode/square\n");
    outdef( b, u, npc, p, wpc, w, e);
  }

  if(vvv==11){ 
    int npc=4;				/* 6 sites per unitcell */
    int wpc=3*npc/2;
		/* base-vectors (koordinate-system) FIX, last 2 not used */
    double b[Nb][2]={ {           1,           0 },
		      {           0,           1 },
                      { cos(M_PI/4), sin(M_PI/4) },
                      { cos(M_PI/4),-sin(M_PI/4) } };
    		/* unitvector of unitcell in units of basevectors FIX */
    int u[2][Nb]={	{  1, 0, 1, 1 },
			{  0, 1, 1,-1 } };
    int p[4][Nb]={			/* R_i of 2 sites in units of base */
			{  0, 0, 0, 0 },
			{  1, 0, 0, 0 },
			{  1, 0, 1, 0 },
			{  1, 1, 1, 0 } };
    int w[4*3/2][5]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc */
      { 0, 1, 1,  0,  0}, /* octagon */
      { 2, 3, 1,  0,  0},
      { 1, 2, 2,  0,  0}, /*   2nd site is -e[0] PBC located */
      { 3, 1, 2,  0,  1},
      { 0, 3, 2, -1, -1},
      { 2, 0, 2,  1,  0}};
    printf("# 4.8.8  J1=octagon J2=octagon/square\n");
    outdef( b, u, npc, p, wpc, w, e);
  }

  return 0;  
}
